#include "app_setup.h"

extern F6_10_PWM_Type p;				// Structure containing start, length and sequence of pwm signals

void vSCT_Init(void)
{
	/* SCT clock source is PL160M_0@120MHz */
	LPC_CCU1->CLK_M3_SCT_CFG = 0x0001;       	/* Enable SCT branch clock in CCU1 on base_m3_clk */
	                                            /* RUN=1, AUTO=0, WAKE=0 */
	LPC_SCT->CONFIG = 0x00000000;               /* Two separate 16-bit timers, clocked internally */
	LPC_SCT->CTRL_L = 0x001C;			   		/* L counter: */
	                                            /* Set prescaler = 1. Clear counter. */
	                                            /* Keep in HALT. Prescaler = 1 */
	                                            /* Bidirectional */
	LPC_SCT->CTRL_H = 0x001C;			  		/* H counter: */
	                                            /* Set prescaler = 1. Clear counter. */
	                                            /* Keep in HALT. Prescaler = 1 */
	                                            /* Bidirectional */

	/* Now use the FSM code to configure the state machine */
	sct_fsm_init();
	
	/* Other manual configuration and overrides: */
    /* Conflict resolution: CLR override for all */
//    LPC_SCT->RES = (2 << (2*0)) | (2 << (2*1)) | (2 << (2*2)) | (2 << (2*3));

    /* Bidirectional Control Register:
     * AH,AL,BH,BL,CH,CL: All are reversed when L counts down. (MAT0...5)
     */
    //LPC_SCT->BIDIRCTRL = (LPC_SCT->BIDIRCTRL & ~0x0000CFFF) | 0x00004555;
	//LPC_SCT->BIDIRCTRL = (LPC_SCT->BIDIRCTRL & ~0x00000FFF) | 0x00000555;

	// OUTPUT0-5 = PWM, OUTPUT8=ADC_TRIG
	// 0x0 Set and clear do not depend on any counter.
	// 0x1 Set and clear are reversed when counter L or the unified counter is counting down.
	// 0x2 Set and clear are reversed when counter H is counting down. Do not use if UNIFY = 1.
	LPC_SCT->OUTPUTDIRCTRL	= (1<<0)|(1<<2)|(1<<4)|(1<<6)|(1<<8)|(1<<10)|(1<<16);
}

/******************************************************************************
** Function name:		vSCT_Start
**
** Descriptions:		Start SCT
**
** Parameters:			None
** Returned value:		None
** 
******************************************************************************/
void vSCT_Start(void)
{
 	/* Start the SCT */
    LPC_SCT->CTRL_U &= ~((1 << 2) | (1 << 18));  /* Run H+L */
}
