/*****************************************************************************
 *   dma.c:  DMA module file for NXP LPC23xx/24xx Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.07.20  ver 1.00    Preliminary version, first Release
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#include "LPC18xx.h"                        /* LPC23xx/24xx definitions */
#include "type.h"
#include "lpc18xx_dma.h"

/* Private variables ---------------------------------------------------------*/
static DMA_CH_CALLBACK DMA_callbacks[8] =
{
	NULL,  	/* CH0 callback */
	NULL,  	/* CH1 callback */
	NULL,  	/* CH2 callback */
	NULL,  	/* CH3 callback */
	NULL,  	/* CH4 callback */
	NULL,  	/* CH5 callback */
	NULL,  	/* CH6 callback */
	NULL,  	/* CH7 callback */
};

/******************************************************************************
** Function name:		DMA_IRQHandler
**
** Descriptions:		DMA interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
******************************************************************************/
uint32_t regVal;

void DMA_IRQHandler(void) 
{ 
	uint8_t i;
	
	regVal = LPC_GPDMA->INTTCSTAT;
	LPC_GPDMA->INTTCCLEAR = regVal;
	
	LPC_GPDMA->INTERRCLR = LPC_GPDMA->INTERRSTAT;

	for(i=0;i<8;i++)
	{
		if(regVal&(1<<i) && DMA_callbacks[i] != NULL)
		{
			(*DMA_callbacks[i])();	
		}
	}
}

void vDMA_Init(void)
{
	LPC_GPDMA->CONFIG = (0x01 << 0);	// Enable DMA controller, little endian 
	LPC_CREG->DMAMUX = (1<<2);			// Selects DMA to peripheral connection for DMA peripheral 1: UART0 transmit 
	NVIC_EnableIRQ(DMA_IRQn);
}

/*****************************************************************************
** Function name:		DMA_SetChCallback
**
** Descriptions:		set DMA interrupt callbacks
**
** Parameters:			DMA channel number and callback function
** Returned value:		None
** 
*****************************************************************************/
void vDMA_SetChCallback(uint8_t ch, DMA_CH_CALLBACK callback)
{
	DMA_callbacks[ch]= callback;
}



/******************************************************************************
**                            End Of File
******************************************************************************/
