/*****************************************************************************
 *   uart.h:  Header file for NXP LPC18xx Family Microprocessors
 *
 *   Copyright(C) 2010, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2010.04.20  ver 1.00    Preliminary version, first Release
 *
******************************************************************************/
#ifndef __UART_H 
#define __UART_H

#define TX_BUFF_MAXSIZE	32*1024 		//(256+(8*701))	// 256 + 8*701 points data buffer

#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80

#define BUFSIZE		0x40

typedef void (* UART_RX_CALLBACK)(uint8_t ch);

/** @brief UART Initialization buffer structure */
typedef struct {
	uint32_t portNum;
	uint32_t baudrate;
	UART_RX_CALLBACK rx_callback;			/*!< Pointer to Call back function when a character is received */					
} UART_Init_Type;

uint32_t u32UARTInit(UART_Init_Type* pInit); 
void UART0_IRQHandler( void );
void UART1_IRQHandler( void );
void vUARTSend( uint32_t portNum, uint8_t *BufferPtr, uint32_t Length );
void vUARTPutc(uint8_t port, uint8_t c);
void vUART0_DMA_Send(uint32_t *data, uint32_t size);

#endif /* end __UART_H */
/*****************************************************************************
**                            End Of File
******************************************************************************/
