/*****************************************************************************
 * $Id: mc_calib.c 6577 2011-06-06 12:00:00Z nxp16893 $
 *
 * Project: MC_lib
 *
 * Description: Motor Control calibration libraries
 *
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 *
 *****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include "mc_type.h"
#include "mc_calib.h"
#include "mc_inc.h"

F6_10 f6_10QEIduty;
uint32_t u32QEIdelay;

void vCALIB_InitQEICalib(F6_10 duty, uint32_t delay)
{
	f6_10QEIduty.full = duty.full;
	u32QEIdelay = delay;
}

void vCALIB_QEICalib(CALIB_Type *calib)
{
	F6_10_PWM_Type pwm;
	int16_t alpha=0; 

	printf("Start QEI calibration\n\r");
  	
	/* Normal direction */
	vQEI_SetDirection(0);			// Set non-inverted QEI direction
	calib->QEIinvert = FALSE; 		// Save direction

	/* Reset index counter */	
	vQEI_ResInx();		
		
	/* Check if index pulse must be used */
	if(calib->QEIuseinx)
	{
		printf("Rotate untill index pulse\n\r");
		
		/* Enable and clear index interrupt */
		vQEI_IntCmd(QEI_INTFLAG_INX_Int, ENABLE); 		
		vQEI_IntClear(QEI_INTFLAG_INX_Int); 
	
		/* Rotate untill QEI index interrupt */
		while(!fsQEI_GetIntStatus(QEI_INTFLAG_INX_Int))
		{
			vDelay_Ms(1);
			f6_10MC_Commutate(f6_10QEIduty.full*32,  alpha);
			alpha += 50;
			if(alpha >= F6_10_CONST(2.0*PI))alpha=0; 
		}
	}
	else
	{
		printf("Rotate to get direction\r");
		
		/* Rotate a bit */
		for(alpha=0;alpha<F6_10_CONST(2.0*PI);alpha+=50)
		{
			vDelay_Ms(1);
			f6_10MC_Commutate(f6_10QEIduty.full*32,  alpha);
		}
	}

	/* Now determine direction */
	if(u8QEI_Direction())
	{
		/* Index counter decremented: direction is inverted */
		printf("Inverted direction\n\r");
		vQEI_SetDirection(1);  		// Set inverted QEI direction
		calib->QEIinvert = TRUE; 	// Save direction
	}
	
	/* Get QEI offset */
	printf("Power phase 1 and measure offset\n\r");
	pwm.A.full = f6_10QEIduty.full;
	pwm.B.full = F6_10_CONST(0.0);
	pwm.C.full = F6_10_CONST(0.0);
	vMC_LIB_SetPWM_F6_10(&pwm); 		// Set PWM on phase 1
	vDelay_Ms(u32QEIdelay);

	if(!(calib->QEIuseinx))
	{
		printf("Reset position\n\r");
		vQEI_ResPos();	// Reset position; set offset to 0
	}

	calib->QEIoffset = LPC_QEI->POS;	// Save QEI offset
  	printf("QEI offset = %d\n\r",calib->QEIoffset);	
	printf("QEI invert = %d\n\r",calib->QEIinvert); 

	calib->QEIcalib = TRUE;
	calib->QEIauto_complete = TRUE;
	return;
}

/*****************************************************************************
**                            End Of File
******************************************************************************/
