/*****************************************************************************
 * $Id: mc_bldc.c 6577 2011-06-06 12:00:00Z nxp16893 $
 *
 * Project: MC_lib
 *
 * Description: Brushless DC Motor Control libraries
 *
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 *
 *****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#include "mc_inc.h"
#include <stdint.h>
#include "mc_foc.h"
#include "mc_bldc.h"
#include "mc_datastorage.h"

uint8_t u8MS_DS_LoadParam(MC_STORAGE_Type *stored)
{
#if USE_EEPROM	
	/* EEPROM read routines tbd */
	return 1;
#elif USE_FLASH
	/* FLASH read routines tbd */
	return 1;
#else
	/* No storage available */
	return 0;
#endif
}

void vMC_DS_ImportStorage(MC_STORAGE_Type *stored, MC_DATA_Type *mc)
{
	/* Import control/status parameters */
	memcpy((uint8_t *)&mc->ctrl, (uint8_t *)&stored->ctrl , sizeof(mc->ctrl));
	/* Import calibration parameters */
	memcpy((uint8_t *)&mc->calib, (uint8_t *)&stored->calib , sizeof(mc->calib));

	/* Import stored control parameters */
	vMC_LIB_PID_F6_10_Copy((F6_10_PID_Type *)&stored->ctrl.pidSpeed, (F6_10_PID_Type *)&mc->ctrl.pidSpeed);
		
#if defined MCLIB_TYPE_BLDC
	/* Import stored BLDC parameters */
	vMC_BLDC_ImportStorage(&stored->var.bldc, (BLDC_Type *)&mc->var.bldc);		
#elif defined MCLIB_TYPE_STEPPER
	/* tbd */
#elif defined MCLIB_TYPE_SVPWM
	/* tbd */
#elif defined MCLIB_TYPE_FOC_SYNC
	/* Import stored FOC parameters */
	vMC_FOC_ImportStorage(&stored->var.foc, (FOC_Type *)&mc->var.foc);	
#elif defined MCLIB_TYPE_FOC_ASYNC
	/* tbd */
#endif
}

/*****************************************************************************
**                            End Of File
******************************************************************************/
